@@ -21,28 +21,34 @@
 #
 """Applications used as examples, for testing, or in simple situations"""
 import pickle
+import random
 import urllib.error
 import urllib.request
 
 import time
 import numpy as np
 
-from .. import droputils, utils
-from ..drop import BarrierAppDROP, BranchAppDrop, ContainerDROP
-from ..meta import dlg_float_param, dlg_string_param
-from ..meta import dlg_bool_param, dlg_int_param
-from ..meta import dlg_component, dlg_batch_input
-from ..meta import dlg_batch_output, dlg_streaming_input
+from dlg import droputils, utils
+from dlg.drop import BarrierAppDROP, BranchAppDrop, ContainerDROP
+from dlg.meta import dlg_float_param, dlg_string_param
+from dlg.meta import dlg_bool_param, dlg_int_param
+from dlg.meta import dlg_component, dlg_batch_input
+from dlg.meta import dlg_batch_output, dlg_streaming_input
 
 from dlg.apps.pyfunc import serialize_data, deserialize_data
 
+
 class NullBarrierApp(BarrierAppDROP):
-    compontent_meta = dlg_component('NullBarrierApp', 'Null Barrier.',
-                                    [dlg_batch_input('binary/*', [])],
-                                    [dlg_batch_output('binary/*', [])],
-                                    [dlg_streaming_input('binary/*')])
+    component_meta = dlg_component(
+        "NullBarrierApp",
+        "Null Barrier.",
+        [dlg_batch_input("binary/*", [])],
+        [dlg_batch_output("binary/*", [])],
+        [dlg_streaming_input("binary/*")],
+    )
 
     """A BarrierAppDrop that doesn't perform any work"""
+
     def run(self):
         pass
 
@@ -61,12 +67,16 @@ class NullBarrierApp(BarrierAppDROP):
 # @par EAGLE_END
 class SleepApp(BarrierAppDROP):
     """A BarrierAppDrop that sleeps the specified amount of time (0 by default)"""
-    compontent_meta = dlg_component('SleepApp', 'Sleep App.',
-                                    [dlg_batch_input('binary/*', [])],
-                                    [dlg_batch_output('binary/*', [])],
-                                    [dlg_streaming_input('binary/*')])
 
-    sleepTime = dlg_float_param('sleep time', 0)
+    component_meta = dlg_component(
+        "SleepApp",
+        "Sleep App.",
+        [dlg_batch_input("binary/*", [])],
+        [dlg_batch_output("binary/*", [])],
+        [dlg_streaming_input("binary/*")],
+    )
+
+    sleepTime = dlg_float_param("sleep time", 0)
 
     def initialize(self, **kwargs):
         super(SleepApp, self).initialize(**kwargs)
@@ -92,10 +102,14 @@ class CopyApp(BarrierAppDROP):
     All inputs are copied into all outputs in the order they were declared in
     the graph.
     """
-    compontent_meta = dlg_component('CopyApp', 'Copy App.',
-                                    [dlg_batch_input('binary/*', [])],
-                                    [dlg_batch_output('binary/*', [])],
-                                    [dlg_streaming_input('binary/*')])
+
+    component_meta = dlg_component(
+        "CopyApp",
+        "Copy App.",
+        [dlg_batch_input("binary/*", [])],
+        [dlg_batch_output("binary/*", [])],
+        [dlg_streaming_input("binary/*")],
+    )
 
     def run(self):
         self.copyAll()
@@ -155,16 +169,20 @@ class RandomArrayApp(BarrierAppDROP):
     high:     float, upper boundary (will be converted to int for integer arrays)
     size:     int, number of array elements
     """
-    compontent_meta = dlg_component('RandomArrayApp', 'Random Array App.',
-                                    [dlg_batch_input('binary/*', [])],
-                                    [dlg_batch_output('binary/*', [])],
-                                    [dlg_streaming_input('binary/*')])
+
+    component_meta = dlg_component(
+        "RandomArrayApp",
+        "Random Array App.",
+        [dlg_batch_input("binary/*", [])],
+        [dlg_batch_output("binary/*", [])],
+        [dlg_streaming_input("binary/*")],
+    )
 
     # default values
-    integer = dlg_bool_param('integer', True)
-    low = dlg_float_param('low', 0)
-    high = dlg_float_param('high', 100)
-    size = dlg_int_param('size', 100)
+    integer = dlg_bool_param("integer", True)
+    low = dlg_float_param("low", 0)
+    high = dlg_float_param("high", 100)
+    size = dlg_int_param("size", 100)
     marray = []
 
     def initialize(self, **kwargs):
@@ -174,8 +192,7 @@ class RandomArrayApp(BarrierAppDROP):
         # At least one output should have been added
         outs = self.outputs
         if len(outs) < 1:
-            raise Exception(
-                'At least one output should have been added to %r' % self)
+            raise Exception("At least one output should have been added to %r" % self)
         self.generateRandomArray()
         for o in outs:
             d = pickle.dumps(self.marray)
@@ -226,16 +243,24 @@ class AverageArraysApp(BarrierAppDROP):
 
     method:  string <['mean']|'median'>, use mean or median as method.
     """
+
     from numpy import mean, median
-    compontent_meta = dlg_component('RandomArrayApp', 'Random Array App.',
-                                    [dlg_batch_input('binary/*', [])],
-                                    [dlg_batch_output('binary/*', [])],
-                                    [dlg_streaming_input('binary/*')])
+
+    component_meta = dlg_component(
+        "RandomArrayApp",
+        "Random Array App.",
+        [dlg_batch_input("binary/*", [])],
+        [dlg_batch_output("binary/*", [])],
+        [dlg_streaming_input("binary/*")],
+    )
 
     # default values
-    methods = ['mean', 'median']
-    method = dlg_string_param('method', methods[0])
-    marray = []
+    methods = ["mean", "median"]
+    method = dlg_string_param("method", methods[0])
+
+    def __init__(self, oid, uid, **kwargs):
+        super().__init__(oid, kwargs)
+        self.marray = []
 
     def initialize(self, **kwargs):
         super(AverageArraysApp, self).initialize(**kwargs)
@@ -245,12 +270,11 @@ class AverageArraysApp(BarrierAppDROP):
 
         outs = self.outputs
         if len(outs) < 1:
-            raise Exception(
-                'At least one output should have been added to %r' % self)
+            raise Exception("At least one output should have been added to %r" % self)
         self.getInputArrays()
-        avg = self.averageArray()
+        self._avg = self.averageArray()
         for o in outs:
-            d = pickle.dumps(avg)
+            d = pickle.dumps(self._avg)
             o.len = len(d)
             o.write(d)  # average across inputs
 
@@ -262,18 +286,23 @@ class AverageArraysApp(BarrierAppDROP):
         """
         ins = self.inputs
         if len(ins) < 1:
-            raise Exception(
-                'At least one input should have been added to %r' % self)
-
-        marray = [pickle.loads(droputils.allDropContents(inp)) for inp in ins]
+            raise Exception("At least one input should have been added to %r" % self)
+        marray = []
+        for inp in ins:
+            sarray = droputils.allDropContents(inp)
+            if len(sarray) == 0:
+                print(f"Input does not contain data!")
+            else:
+                sarray = pickle.loads(sarray)
+                marray.extend(sarray)
         self.marray = marray
 
-
     def averageArray(self):
 
         method_to_call = getattr(np, self.method)
         return method_to_call(self.marray, axis=0)
 
+
 ##
 # @brief HelloWorldApp
 # @details A simple APP that implements the standard Hello World in DALiuGE.
@@ -297,35 +326,40 @@ class HelloWorldApp(BarrierAppDROP):
     Keywords:
     greet:   string, [World], whom to greet.
     """
-    compontent_meta = dlg_component('HelloWorldApp', 'Hello World App.',
-                                    [dlg_batch_input('binary/*', [])],
-                                    [dlg_batch_output('binary/*', [])],
-                                    [dlg_streaming_input('binary/*')])
 
-    greet = dlg_string_param('greet', 'World')
+    component_meta = dlg_component(
+        "HelloWorldApp",
+        "Hello World App.",
+        [dlg_batch_input("binary/*", [])],
+        [dlg_batch_output("binary/*", [])],
+        [dlg_streaming_input("binary/*")],
+    )
+
+    greet = dlg_string_param("greet", "World")
 
     def run(self):
         ins = self.inputs
         # if no inputs use the parameter else use the input
         if len(ins) == 0:
-            self.greeting = 'Hello %s' % self.greet
+            self.greeting = "Hello %s" % self.greet
         elif len(ins) != 1:
-            raise Exception(
-                'Only one input expected for %r' % self)
-        else: # the input is expected to be a vector. We'll use the first element
-            self.greeting = 'Hello %s' % str(pickle.loads(droputils.allDropContents(ins[0]))[0])
+            raise Exception("Only one input expected for %r" % self)
+        else:  # the input is expected to be a vector. We'll use the first element
+            self.greeting = "Hello %s" % str(
+                pickle.loads(droputils.allDropContents(ins[0]))[0]
+            )
 
         outs = self.outputs
         if len(outs) < 1:
-            raise Exception(
-                'At least one output should have been added to %r' % self)
+            raise Exception("At least one output should have been added to %r" % self)
         for o in outs:
             o.len = len(self.greeting.encode())
             o.write(self.greeting.encode())  # greet across all outputs
 
+
 ##
 # @brief UrlRetrieveApp
-# @details A simple APP that retrieves the content of a URL and writes.
+# @details A simple APP that retrieves the content of a URL and writes
 # it to all outputs.
 # @par EAGLE_START
 # @param category PythonApp
@@ -343,12 +377,16 @@ class UrlRetrieveApp(BarrierAppDROP):
     Keywords:
     URL:   string, URL to retrieve.
     """
-    compontent_meta = dlg_component('UrlRetrieveApp', 'URL Retrieve App',
-                                    [dlg_batch_input('binary/*', [])],
-                                    [dlg_batch_output('binary/*', [])],
-                                    [dlg_streaming_input('binary/*')])
 
-    url = dlg_string_param('url', '')
+    component_meta = dlg_component(
+        "UrlRetrieveApp",
+        "URL Retrieve App",
+        [dlg_batch_input("binary/*", [])],
+        [dlg_batch_output("binary/*", [])],
+        [dlg_streaming_input("binary/*")],
+    )
+
+    url = dlg_string_param("url", "")
 
     def run(self):
         try:
@@ -360,12 +398,12 @@ class UrlRetrieveApp(BarrierAppDROP):
 
         outs = self.outputs
         if len(outs) < 1:
-            raise Exception(
-                'At least one output should have been added to %r' % self)
+            raise Exception("At least one output should have been added to %r" % self)
         for o in outs:
             o.len = len(content)
             o.write(content)  # send content to all outputs
 
+
 ##
 # @brief GenericScatterApp
 # @details An APP that splits about any object that can be converted to a numpy array
@@ -386,10 +424,14 @@ class GenericScatterApp(BarrierAppDROP):
     An APP that splits an object that has a len attribute into <numSplit> parts and
     returns a numpy array of arrays, where the first axis is of length <numSplit>.
     """
-    compontent_meta = dlg_component('GenericScatterApp', 'Scatter an array like object into numSplit parts',
-                                    [dlg_batch_input('binary/*', [])],
-                                    [dlg_batch_output('binary/*', [])],
-                                    [dlg_streaming_input('binary/*')])
+
+    component_meta = dlg_component(
+        "GenericScatterApp",
+        "Scatter an array like object into numSplit parts",
+        [dlg_batch_input("binary/*", [])],
+        [dlg_batch_output("binary/*", [])],
+        [dlg_streaming_input("binary/*")],
+    )
 
     def initialize(self, **kwargs):
         super(GenericScatterApp, self).initialize(**kwargs)
@@ -417,11 +459,12 @@ class GenericScatterApp(BarrierAppDROP):
             o.len = len(d)
             o.write(d)  # average across inputs
 
+
 class SimpleBranch(BranchAppDrop, NullBarrierApp):
     """Simple branch app that is told the result of its condition"""
 
     def initialize(self, **kwargs):
-        self.result = self._getArg(kwargs, 'result', True)
+        self.result = self._getArg(kwargs, "result", True)
         BranchAppDrop.initialize(self, **kwargs)
 
     def run(self):
@@ -429,3 +472,66 @@ class SimpleBranch(BranchAppDrop, NullBarrierApp):
 
     def condition(self):
         return self.result
+
+
+
+##
+# @brief ListAppendThrashingApp\n
+# @details A testing APP that appends a random integer to a list num times.
+# This is a CPU intensive operation and can thus be used to provide a test for application threading
+# since this operation will not yield.
+# The resulting array will be sent to all connected output apps.
+# @par EAGLE_START
+# @param gitrepo $(GIT_REPO)
+# @param version $(PROJECT_VERSION)
+# @param category PythonApp
+# @param[in] param/size/100/Integer/readwrite
+#     \~English the size of the array\n
+# @param[in] param/appclass/dlg.apps.simple.ListAppendThrashingApp/String/readonly
+#     \~English Application class\n
+# @param[out] port/array
+#     \~English Port carrying the random array.
+# @par EAGLE_END
+class ListAppendThrashingApp(BarrierAppDROP):
+    """
+    A BarrierAppDrop that appends random integers to a list N times. It does
+    not require any inputs and writes the generated array to all of its
+    outputs.
+
+    Keywords:
+
+    size:     int, number of array elements
+    """
+    compontent_meta = dlg_component('ListAppendThrashingApp', 'List Append Thrashing',
+                                    [dlg_batch_input('binary/*', [])],
+                                    [dlg_batch_output('binary/*', [])],
+                                    [dlg_streaming_input('binary/*')])
+
+    def initialize(self, **kwargs):
+        self.size = self._getArg(kwargs, 'size', 100)
+        self.marray = []
+        super(ListAppendThrashingApp, self).initialize(**kwargs)
+
+    def run(self):
+        # At least one output should have been added
+        outs = self.outputs
+        if len(outs) < 1:
+            raise Exception(
+                'At least one output should have been added to %r' % self)
+        self.marray = self.generateArray()
+        for o in outs:
+            d = pickle.dumps(self.marray)
+            o.len = len(d)
+            o.write(pickle.dumps(self.marray))
+
+    def generateArray(self):
+        # This operation is wasteful to simulate an N^2 operation.
+        marray = []
+        for _ in range(int(self.size)):
+            marray = []
+            for i in range(int(self.size)):
+                marray.append(random.random())
+        return marray
+
+    def _getArray(self):
+        return self.marray
