@@ -20,25 +20,31 @@
 #    MA 02111-1307  USA
 #
 from abc import abstractmethod, ABCMeta
+from overrides import overrides
 import io
 import logging
 import os
+import sys
 import urllib.parse
-
-from typing import Optional
+from abc import abstractmethod, ABCMeta
+from typing import Optional, Union
 
 from . import ngaslite
 from .apps.plasmaflight import PlasmaFlightClient
 
 import pyarrow
 import pyarrow.plasma as plasma
+if sys.version_info >= (3, 8):
+    from dlg.shared_memory import DlgSharedMemory
 
 
 logger = logging.getLogger(__name__)
 
+
 class OpenMode:
     OPEN_WRITE, OPEN_READ = range(2)
 
+
 class DataIO(object):
     """
     A class used to read/write data stored in a particular kind of storage in an
@@ -77,9 +83,9 @@ class DataIO(object):
         Writes `data` into the storage
         """
         if self._mode is None:
-            raise ValueError('Writing operation attempted on closed DataIO object')
+            raise ValueError("Writing operation attempted on closed DataIO object")
         if self._mode == OpenMode.OPEN_READ:
-            raise ValueError('Writing operation attempted on write-only DataIO object')
+            raise ValueError("Writing operation attempted on write-only DataIO object")
         return self._write(data, **kwargs)
 
     def read(self, count, **kwargs):
@@ -87,9 +93,9 @@ class DataIO(object):
         Reads `count` bytes from the underlying storage.
         """
         if self._mode is None:
-            raise ValueError('Reading operation attempted on closed DataIO object')
+            raise ValueError("Reading operation attempted on closed DataIO object")
         if self._mode == OpenMode.OPEN_WRITE:
-            raise ValueError('Reading operation attempted on write-only DataIO object')
+            raise ValueError("Reading operation attempted on write-only DataIO object")
         return self._read(count, **kwargs)
 
     def close(self, **kwargs):
@@ -104,7 +110,7 @@ class DataIO(object):
 
     def size(self, **kwargs):
         """
-        Returns the current total size of the underlying stored object. If the 
+        Returns the current total size of the underlying stored object. If the
         storage class does not support this it is supposed to return -1.
         """
         return _size(self)
@@ -126,19 +132,33 @@ class DataIO(object):
         """
 
     @abstractmethod
-    def _open(self, **kwargs): pass
+    def buffer(self) -> Union[memoryview, bytes, bytearray, pyarrow.Buffer]:
+        """
+        Gets a buffer protocol compatible object of the drop data.
+        This may be a zero-copy view of the data or a copy depending
+        on whether the drop stores data in cpu memory or not.
+        """
 
     @abstractmethod
-    def _read(self, count, **kwargs): pass
+    def _open(self, **kwargs):
+        pass
 
     @abstractmethod
-    def _write(self, data, **kwargs): pass
+    def _read(self, count, **kwargs):
+        pass
 
     @abstractmethod
-    def _close(self, **kwargs): pass
+    def _write(self, data, **kwargs):
+        pass
 
     @abstractmethod
-    def _size(self, **kwargs): pass
+    def _close(self, **kwargs):
+        pass
+
+    @abstractmethod
+    def _size(self, **kwargs):
+        pass
+
 
 class NullIO(DataIO):
     """
@@ -169,6 +189,7 @@ class NullIO(DataIO):
     def delete(self):
         pass
 
+
 class ErrorIO(DataIO):
     """
     An DataIO method that throws exceptions if any of its methods is invoked
@@ -195,13 +216,16 @@ class ErrorIO(DataIO):
     def delete(self):
         raise NotImplementedError()
 
+
 class MemoryIO(DataIO):
     """
     A DataIO class that reads/write from/into the BytesIO object given at
     construction time
     """
+    _desc: io.BytesIO  # TODO: This might actually be a problem
 
     def __init__(self, buf: io.BytesIO, **kwargs):
+        super().__init__()
         self._buf = buf
 
     def _open(self, **kwargs):
@@ -235,13 +259,79 @@ class MemoryIO(DataIO):
     def delete(self):
         self._buf.close()
 
-class FileIO(DataIO):
+    @overrides
+    def buffer(self) -> memoryview:
+        #  TODO: This may also be an issue
+        return self._open().getbuffer()
 
+
+
+class SharedMemoryIO(DataIO):
+    """
+    A DataIO class that writes to a shared memory buffer
+    """
+
+    def __init__(self, uid, session_id, **kwargs):
+        super().__init__()
+        self._name = f'{session_id}_{uid}'
+        self._written = 0
+        self._pos = 0
+        self._buf = None
+
+    def _open(self, **kwargs):
+        self._pos = 0
+        if self._buf is None:
+            if self._mode == OpenMode.OPEN_WRITE:
+                self._written = 0
+            self._buf = DlgSharedMemory(self._name)
+        return self._buf
+
+    def _write(self, data, **kwargs):
+        total_size = len(data) + self._written
+        if total_size > self._buf.size:
+            self._buf.resize(total_size)
+            self._buf.buf[self._written:total_size] = data
+            self._written = total_size
+        else:
+            self._buf.buf[self._written:total_size] = data
+            self._written = total_size
+            self._buf.resize(total_size)
+            """
+            It may be inefficient to resize many times, but assuming data is written 'once' this is
+            might be tolerable and guarantees that the size of the underlying buffer is tight.
+            """
+        return len(data)
+
+    def _read(self, count=4096, **kwargs):
+        if self._pos == self._buf.size:
+            return None
+        start = self._pos
+        end = self._pos + count
+        end = min(end, self._buf.size)
+        out = self._buf.buf[start:end]
+        self._pos = end
+        return out
+
+    def _close(self, **kwargs):
+        if self._mode == OpenMode.OPEN_WRITE:
+            self._buf.resize(self._written)
+        self._buf.close()
+        self._buf = None
+
+    def exists(self):
+        return self._buf is not None
+
+    def delete(self):
+        self._close()
+
+
+class FileIO(DataIO):
+    _desc: io.BufferedReader
     def __init__(self, filename, **kwargs):
         super(FileIO, self).__init__()
         self._fnm = filename
 
-    def _open(self, **kwargs):
+    def _open(self, **kwargs) -> io.BufferedReader:
         flag = 'r' if self._mode is OpenMode.OPEN_READ else 'w'
         flag += 'b'
         try:
@@ -272,15 +362,28 @@ class FileIO(DataIO):
     def delete(self):
         os.unlink(self._fnm)
 
+    @overrides
+    def buffer(self) -> bytes:
+        return self._desc.read(-1)
+
 
 class NgasIO(DataIO):
-    '''
+    """
     A DROP whose data is finally stored into NGAS. Since NGAS doesn't
     support appending data to existing files, we store all the data temporarily
     in a file on the local filesystem and then move it to the NGAS destination
-    '''
+    """
 
-    def __init__(self, hostname, fileId, port = 7777, ngasConnectTimeout=2, ngasTimeout=2, length=-1, mimeType='application/octet-stream'):
+    def __init__(
+        self,
+        hostname,
+        fileId,
+        port=7777,
+        ngasConnectTimeout=2,
+        ngasTimeout=2,
+        length=-1,
+        mimeType="application/octet-stream",
+    ):
 
         # Check that we actually have the NGAMS client libraries
         try:
@@ -290,17 +393,20 @@ class NgasIO(DataIO):
             raise
 
         super(NgasIO, self).__init__()
-        self._ngasSrv            = hostname
-        self._ngasPort           = port
+        self._ngasSrv = hostname
+        self._ngasPort = port
         self._ngasConnectTimeout = ngasConnectTimeout
-        self._ngasTimeout        = ngasTimeout
-        self._fileId             = fileId
-        self._length             = length
-        self._mimeType           = mimeType
+        self._ngasTimeout = ngasTimeout
+        self._fileId = fileId
+        self._length = length
+        self._mimeType = mimeType
 
     def _getClient(self):
         from ngamsPClient import ngamsPClient  # @UnresolvedImport
-        return ngamsPClient.ngamsPClient(self._ngasSrv, self._ngasPort, self._ngasTimeout)
+
+        return ngamsPClient.ngamsPClient(
+            self._ngasSrv, self._ngasPort, self._ngasTimeout
+        )
 
     def _open(self, **kwargs):
         if self._mode == OpenMode.OPEN_WRITE:
@@ -308,7 +414,7 @@ class NgasIO(DataIO):
             # request with data. Thus the only way we can currently archive data
             # into NGAS is by accumulating it all on our side and finally
             # sending it over.
-            self._buf = b''
+            self._buf = b""
             self._writtenDataSize = 0
         return self._getClient()
 
@@ -316,10 +422,15 @@ class NgasIO(DataIO):
         client = self._desc
         if self._mode == OpenMode.OPEN_WRITE:
             reply, msg, _, _ = client._httpPost(
-                     client.getHost(), client.getPort(), 'QARCHIVE',
-                     self._mimeType, dataRef=self._buf,
-                     pars=[['filename',self._fileId]], dataSource='BUFFER',
-                     dataSize=self._writtenDataSize)
+                client.getHost(),
+                client.getPort(),
+                "QARCHIVE",
+                self._mimeType,
+                dataRef=self._buf,
+                pars=[["filename", self._fileId]],
+                dataSource="BUFFER",
+                dataSize=self._writtenDataSize,
+            )
             self._buf = None
             if reply != 200:
                 # Probably msg is not enough, we need to unpack the status XML doc
@@ -341,50 +452,71 @@ class NgasIO(DataIO):
 
     def exists(self):
         import ngamsLib  # @UnresolvedImport
-        status = self._getClient().sendCmd('STATUS', pars=[['fileId', self._fileId]])
+
+        status = self._getClient().sendCmd("STATUS", pars=[["fileId", self._fileId]])
         return status.getStatus() == ngamsLib.ngamsCore.NGAMS_SUCCESS
 
     def fileStatus(self):
         import ngamsLib  # @UnresolvedImport
-        #status = self._getClient().sendCmd('STATUS', pars=[['fileId', self._fileId]])
-        status = self._getClient.fileStatus('STATUS?file_id=%s' % self._fileId)
+
+        # status = self._getClient().sendCmd('STATUS', pars=[['fileId', self._fileId]])
+        status = self._getClient.fileStatus("STATUS?file_id=%s" % self._fileId)
         if status.getStatus() != ngamsLib.ngamsCore.NGAMS_SUCCESS:
             raise FileNotFoundError
-        fs = dict(status.getDiskStatusList()[0].getFileObjList()[0].genXml().attributes.items())
+        fs = dict(
+            status.getDiskStatusList()[0]
+            .getFileObjList()[0]
+            .genXml()
+            .attributes.items()
+        )
         return fs
 
-
     def delete(self):
-        pass # We never delete stuff from NGAS
+        pass  # We never delete stuff from NGAS
+
 
 class NgasLiteIO(DataIO):
-    '''
+    """
     An IO class whose data is finally stored into NGAS. It uses the ngaslite
     module of DALiuGE instead of the full client-side libraries provided by NGAS
     itself, since they might not be installed everywhere.
 
     The `ngaslite` module doesn't support the STATUS command yet, and because of
     that this class will throw an error if its `exists` method is invoked.
-    '''
+    """
 
-    def __init__(self, hostname, fileId, port=7777, ngasConnectTimeout=2, ngasTimeout=2, length=-1, \
-        mimeType='application/octet-stream'):
+    def __init__(
+        self,
+        hostname,
+        fileId,
+        port=7777,
+        ngasConnectTimeout=2,
+        ngasTimeout=2,
+        length=-1,
+        mimeType="application/octet-stream",
+    ):
         super(NgasLiteIO, self).__init__()
-        self._ngasSrv            = hostname
-        self._ngasPort           = port
+        self._ngasSrv = hostname
+        self._ngasPort = port
         self._ngasConnectTimeout = ngasConnectTimeout
-        self._ngasTimeout        = ngasTimeout
-        self._fileId             = fileId
-        self._length             = length
-        self._mimeType           = mimeType
+        self._ngasTimeout = ngasTimeout
+        self._fileId = fileId
+        self._length = length
+        self._mimeType = mimeType
 
     def _is_length_unknown(self):
         return self._length is None or self._length < 0
 
     def _getClient(self):
         return ngaslite.open(
-            self._ngasSrv, self._fileId, port=self._ngasPort, timeout=self._ngasTimeout,\
-                mode=self._mode, mimeType=self._mimeType, length=self._length)
+            self._ngasSrv,
+            self._fileId,
+            port=self._ngasPort,
+            timeout=self._ngasTimeout,
+            mode=self._mode,
+            mimeType=self._mimeType,
+            length=self._length,
+        )
 
     def _open(self, **kwargs):
         if self._mode == OpenMode.OPEN_WRITE:
@@ -393,7 +525,7 @@ class NgasLiteIO(DataIO):
                 # of the whole fileObject to be known and sent up-front as part of the header. Thus
                 # is size is not provided all data will be buffered IN MEMORY and only sent to NGAS
                 # when finishArchive is called.
-                self._buf = b''
+                self._buf = b""
                 self._writtenDataSize = 0
         return self._getClient()
 
@@ -402,13 +534,16 @@ class NgasLiteIO(DataIO):
             conn = self._desc
             if self._is_length_unknown():
                 # If length wasn't known up-front we first send Content-Length and then the buffer here.
-                conn.putheader('Content-Length', len(self._buf))
+                conn.putheader("Content-Length", len(self._buf))
                 conn.endheaders()
                 logger.debug("Sending data for file %s to NGAS" % (self._fileId))
                 conn.send(self._buf)
                 self._buf = None
             else:
-                logger.debug("Length is known, assuming data has been sent (%s, %s)" % (self.fileId, self._length))
+                logger.debug(
+                    "Length is known, assuming data has been sent (%s, %s)"
+                    % (self.fileId, self._length)
+                )
             ngaslite.finishArchive(conn, self._fileId)
             conn.close()
         else:
@@ -434,7 +569,8 @@ class NgasLiteIO(DataIO):
         return ngaslite.fileStatus(self._ngasSrv, self._ngasPort, self._fileId)
 
     def delete(self):
-        pass # We never delete stuff from NGAS
+        pass  # We never delete stuff from NGAS
+
 
 def IOForURL(url):
     """
@@ -443,18 +579,21 @@ def IOForURL(url):
     """
     url = urllib.parse.urlparse(url)
     io = None
-    if url.scheme == 'file':
+    if url.scheme == "file":
         hostname = url.netloc
         filename = url.path
-        if hostname == 'localhost' or hostname == '127.0.0.1' or \
-           hostname == os.uname()[1]:
+        if (
+            hostname == "localhost"
+            or hostname == "127.0.0.1"
+            or hostname == os.uname()[1]
+        ):
             io = FileIO(filename)
-    elif url.scheme == 'null':
+    elif url.scheme == "null":
         io = NullIO()
-    elif url.scheme == 'ngas':
+    elif url.scheme == "ngas":
         networkLocation = url.netloc
-        if ':' in networkLocation:
-            hostname, port = networkLocation.split(':')
+        if ":" in networkLocation:
+            hostname, port = networkLocation.split(":")
             port = int(port)
         else:
             hostname = networkLocation
@@ -463,17 +602,16 @@ def IOForURL(url):
         try:
             io = NgasIO(hostname, fileId, port)
         except:
-            logger.warning('NgasIO not available, using NgasLiteIO instead')
+            logger.warning("NgasIO not available, using NgasLiteIO instead")
             io = NgasLiteIO(hostname, fileId, port)
 
-    logger.debug('I/O chosen for dataURL %s: %r', url, io)
+    logger.debug("I/O chosen for dataURL %s: %r", url, io)
 
     return io
 
 
 class PlasmaIO(DataIO):
-
-    def __init__(self, object_id, plasma_path='/tmp/plasma'):
+    def __init__(self, object_id, plasma_path="/tmp/plasma"):
         super(PlasmaIO, self).__init__()
         self._plasma_path = plasma_path
         self._object_id = object_id
@@ -509,14 +647,20 @@ class PlasmaIO(DataIO):
     def delete(self):
         pass
 
+    @overrides
+    def buffer(self) -> memoryview:
+        [data] = self._desc.get_buffers([self._object_id])
+        return memoryview(data)
+
 
 class PlasmaFlightIO(DataIO):
     def __init__(
-            self,
-            object_id: plasma.ObjectID,
-            plasma_path='/tmp/plasma',
-            flight_path: Optional[str] = None,
-            size: int = -1):
+        self,
+        object_id: plasma.ObjectID,
+        plasma_path="/tmp/plasma",
+        flight_path: Optional[str] = None,
+        size: int = -1,
+    ):
         super(PlasmaFlightIO, self).__init__()
         assert size >= -1
         self._object_id = object_id
@@ -551,7 +695,9 @@ class PlasmaFlightIO(DataIO):
         if not self._writer:
             if self._size == -1:
                 # stream into resizeable buffer
-                logger.warning("Using dynamically sized Plasma buffer. Performance may be reduced.")
+                logger.warning(
+                    "Using dynamically sized Plasma buffer. Performance may be reduced."
+                )
                 self._writer = pyarrow.BufferOutputStream()
             else:
                 # optimally write directly to fixed size plasma buffer
@@ -564,4 +710,8 @@ class PlasmaFlightIO(DataIO):
         return self._desc.exists(self._object_id, self._flight_path)
 
     def delete(self):
-        pass
\ No newline at end of file
+        pass
+
+    @overrides
+    def buffer(self) -> memoryview:
+        return self._desc.get(self._object_id, self._flight_path)
